//*******************************************************************************
//  MSP-FET430F123 Epson CX4200 Paper Emulator 
//  Written by: Volkan Sahin
//  First Edit: May 29, 2006
//  Version: Beta
//  This software is under the GNU Public License (GPL)
//  I am not responsible for any damage that happens to your printer by using this software.
//  Use at your own risk.
//  Built with MSPGCC
//  Pressing SW2 for a long time (~ 3 secs) activates relay and Led201 starts to blink. 
//  I added  this for future use, such as to turn off the Y-axis motor.  
//  Pressing SW2 for a short period (~1 sec) and then releasing activates paper emulator. 
//  XENC0/XENC1 quadrature inputs of X-axis encoder (on inkjet head)
//  YENC0/YENC1 quadrature inputs of Y-axis encoder (on mainboard) 
//  TRAYHOME tray slot detector input
//  PDETECT Paper sensor (on inkjet head) connection.
//  PSWITCH Paper end sensor connection 
//  Connect PDETECT/PSWITCH signals in parallel to sensors on printer.
//  You can find unused variables, sorry for that, feel free to remove them.  
//******************************************************************************
#include <io.h>
#include <signal.h>
#include <stdio.h>
#include  <msp430x12x.h>
#include <string.h>
#define SYSTEMCLK  8000000
#define BAUDRATE   38400

#define TICKRATE 10000
#define CLKOUT 1000000   //Ref. Osc of MSP 8MHz & TIMER Prescaler set to 8 
#define CAPTURECONSTANT 37500 //300ms 



#define TICKCONSTANT 10 
#define UARTRXTIMEOUT 100 

#define  MASK_10  0x369   
#define  MASK_10B 0x96 


//Port 1 Connections

#define dummy  0x10    // P1.4   IO Mode = Output
#define SWIN  0x08  // P1.3   IO Mode =  INPUT 
#define MSM 0x04 // P1.2   IO Mode = Input 
#define MSP 0x02  // P1.1  IO Mode = Input

//Port 2 Connections
#define NC1    0x80  // P2.7   IO Mode = Output
#define NC0    0x40  // P2.6   IO Mode = Output
#define XENC1  0x20 // P2.5   IO Mode = INPUT 
#define YENC1  0x10  // P2.4   IO Mode =  INPUT
#define XENC0  0x08  // P2.3   IO Mode =  INPUT 
#define YENC0  0x04 // P2.2   IO Mode =  INPUT 
#define PSWITCH 0x02  // P2.1   IO Mode =  INPUT 
#define PDETECT 0x01   // P2.0   IO Mode =  INPUT 

//Port 3 Connections
#define TRAYHOME  0x10  // P3.4   IO Mode = Input 
#define RLY 0x20 // P3.5 IO Mode = Output
#define LED201 0x08  // P3.3   IO Mode = Output


//#define genclk   P1OUT ^= SCK; _NOP(); _NOP();_NOP(); _NOP(); P1OUT ^= SCK; _NOP(); _NOP();_NOP(); _NOP()

#define EVENT_PATTERN 0x01
#define EVENT_TIMER_EXPIRED 0x02
#define EVENT_SIZE_INFO 0x04
#define EVENT_TX_EMPTY  0x08
#define EVENT_UART_TX_EMPTY 0x10
#define EVENT_UART_RX_FULL 0x20
#define EVENT_UART_RX_TIMEOUT 0x40
#define EVENT_NEXT_WORD 0x80
//#define PPOS  0x9f0+0x4f8 //old setting
//#define PPOS  0xe0e
#define PPOS  0xd90
#define PPOSLOWER  0xd26
//PPOS range=0xd26 to 0xef7;mid point= 0xe0e

static volatile unsigned int timetick;
static volatile unsigned int bitcounter;
static volatile unsigned char Event_Status;
static volatile unsigned char i,wdtcnt=32;

static volatile unsigned char start = 0;
static volatile unsigned char clk,addr=0,data,bit_data=0;
static volatile unsigned char  state=0;
static volatile unsigned int totalclk=0;
//static volatile unsigned char *ptr,timercnt = 10;
static volatile unsigned char *ptr;
//volatile unsigned char startsw=0, swprev,blocksw=0,swblocktimer=0,blinktimer=0,blink=0;
volatile unsigned char startsw=0, swprev,blocksw=0,blink=0,ysensepos=0;
volatile unsigned int swblocktimer=0,blinktimer=0,timercnt=10000;
volatile unsigned char cnt=0, limit,paperdetect_prev=1, psw_prev=0;
volatile unsigned long xenc=0,swxpos,swypos;
volatile unsigned char swdetected=0, xsign=0,ysign=0;
volatile unsigned long ppos = PPOS, trayhome=0, yenc=0,yenc1,diff=0;

const unsigned char ysense[4] = {YENC0|YENC1,YENC0&~YENC1,~YENC0&~YENC1,~YENC0&YENC1} ; 
void msp_init_port(void)
{
  //Port 1 Staff
  P1OUT = 0x00;  // Initilize Port 1 
  P1DIR = 0xF7;  // 1111 0111 
  P1SEL = 0x00;  // Select Port
  //Port 2 Staff
  P2OUT = 0x00|PDETECT;  
  P2DIR = 0xC0|PSWITCH;   
  P2SEL = 0x00;  

  //Port 3 Staff
  P3OUT = 0x00;  // Initilize Port 3 
  P3DIR = 0xEF;  // 1110 1111 
  P3SEL = 0x00;  // Select Port

  BCSCTL1 =  XTS + RSEL2 + RSEL1 + RSEL0 ; // DCO Max bias
  BCSCTL2 = SELM1 + SELS+ DIVS1 + DIVS0; //0x00
  DCOCTL = 0xe0;
  
  
  TACTL = ID1 + ID0 + TASSEL1 + TACLR;              // SMCLK, clear TAR
  CCR0 = CAPTURECONSTANT;
  TACTL |= MC0;                         // Start Timer_A in up mode
  CCTL0 = CCIE;                         // CCR0 interrupt enabled
}/*msp_init_port*/

// Timer A0 interrupt service routine
interrupt (TIMERA0_VECTOR) Timer_A(void)
{

   if(!(--timercnt)){
     Event_Status |= EVENT_TIMER_EXPIRED; //Inform Event
     timercnt = 10;
   }//if
   if(!(P1IN&SWIN)){
     swblocktimer++;
     if(swblocktimer >=10) swblocktimer =10;
   }//if
   else{
     if(swprev != (P1IN&SWIN)){
       if(swblocktimer >=10){ 
         swblocktimer=0;
         blocksw = 1;
         startsw = 0;
         blink = 1;
       }//if 
       else{
         if(startsw){
           P3OUT &= ~LED201;
           startsw = 0;
         }//if
         else{
           swblocktimer=0;
           blocksw = 0;
           blink = 0;
           startsw = 1;
           P3OUT |= LED201;
         }//if
       }//if
     }//if
   }//if
   swprev = P1IN&SWIN;
   if(blink){
     if(blinktimer) blinktimer --;
     else {
       if(P3OUT&LED201) P3OUT &= ~LED201;
       else P3OUT |= LED201;
       blinktimer =0; 
     }//if 
   }//if
}// Timer A0 interrupt service routine

interrupt (PORT2_VECTOR) data_extract(void)
{
//   P3OUT |= LED201;
   if(P2IFG&XENC0){ 
     if(!xenc) xenc = 1;
     if(P2IN&XENC1){
       xenc++;
       xsign = 1;
     }//if 
     else{ 
       if(xenc) xenc--;
       xsign = 0;
       yenc1=yenc;
     }//if
   }//if
   if(P2IFG&YENC0){ 
     if(!yenc) yenc = 1;
     if(P2IN&YENC1){
       yenc++;
       ysign = 1;
       trayhome = 0;
     }//if
     else{
       if(yenc) yenc--;
       ysign = 0;
       if(!trayhome) ppos = PPOS - 400;
     }//if
   }//if
   if(yenc>0x9f0 && yenc <0x5000){
     swdetected=1;
     P2OUT |= PSWITCH;
     P2DIR |= PSWITCH;
     if(!ysign && !(P3IN&TRAYHOME) && !trayhome){
       ppos = trayhome = yenc-50;
     }//if
   }//if
   else {
     trayhome = 0;
     swdetected=0;
     ppos =PPOS;
     P2OUT &= ~PSWITCH;
     P2DIR |= PSWITCH;
   }//if
//PPOS-450 to PPOS+15 are valid
   //old if( swdetected&&(xenc <0x707) &&(xenc>0xf4) && (yenc >PPOS+15) ) {
   if( swdetected&&(xenc <0x707) &&(xenc>0xf4) && (yenc > ppos) ) {
     P2OUT &= ~PDETECT;
     P2DIR |= PDETECT;
   }//if
   else{
     P2OUT |= PDETECT;
     P2DIR |= PDETECT;
   }//if
 //  }//if
   psw_prev =P2IN&PSWITCH; 
   paperdetect_prev = P2IN&PDETECT; 
   timercnt = 50; 
//   TACTL |= TACLR;              // clear TAR
   P2IFG = 0x0;//Clear Interrupt flags of Port2
//   P3OUT &= ~LED201;
}//data_extract

// Watchdog Timer interrupt service routine
interrupt (WDT_VECTOR)  watchdog_timer(void)
{
  WDTCTL = WDTPW + WDTHOLD;         // Stop WDT
  _BIC_SR_IRQ(CPUOFF);//Wakeup  CPU
}//watchdog_timer

// USARTinterrupt service routine

void init_usart(void)
{
  U0CTL = CHAR + SWRST;
  U0TCTL = SSEL0 + TXEPT ; //Clock Source = ACLK
  U0BR1 = 0x00; //Baud Rate = 9600 bits/sec
  U0BR0 = 0xd0;
  U0MCTL = 0x00;

  ME2 |= UTXE0 + URXE0;                 // Enabled USART0 TXD/RXD
  U0CTL = CHAR;
}//init_usart  
void delay(cnt)
{
  int i;
  for(i=0;i<cnt;i++);
}


int main(void)
{ 
  unsigned char res,seq=0;
  
  msp_init_port();
  Event_Status =0;
  P2IES = 0x00;
  P2IE = YENC0|XENC0;//Enable Port2 YENC0 & XENC0 interrupts
  P2IFG = 0x0;
  WDTCTL = WDTPW + WDTHOLD;         // Stop WDT
  msp_init_port();
  swprev = P1IN&SWIN;
  psw_prev = P2IN&PSWITCH ;
  paperdetect_prev = P2IN&PDETECT; 
  BCSCTL1 |= XTS;                     
    // Wait for crystal
  do {
   IFG1 &= ~OFIFG;
   delay(255);                     // Time for flag to set
   msp_init_port();
  } while (IFG1 & OFIFG);
  _EINT();                              // Enable Interrupts
  while(1){
    IFG1 &= ~OFIFG; 
    if(blocksw){
      P2OUT = P2IN;
      P2OUT |= YENC0 + YENC1; 
      P3OUT |= RLY;
      P2DIR |= YENC0 + YENC1;
    }//if
    else {
      if(!startsw){ 
        P2DIR &= ~(YENC1 + YENC0);;
        timercnt = 10;
        swdetected = 0;
        yenc=0;
        xenc=0;
        P2DIR = 0;
        P3OUT &= ~RLY;
      }//if    
      else{ 
        P2DIR &= ~(YENC1 + YENC0);;
        P3OUT &= ~RLY;
        if((Event_Status & EVENT_TIMER_EXPIRED)){ 
          swdetected = 0;
          P2OUT &= ~PSWITCH;
          P2OUT |= PDETECT;
          P2DIR |= PSWITCH|PDETECT;
          Event_Status &= ~EVENT_TIMER_EXPIRED; 
        }//if
      }//if
    }//if
  }//while
}

